"""Predict reference period."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import logging

# externals
import torch
import numpy as np
import xarray as xr
from torch.utils.data import DataLoader

# locals
from climax.core.dataset import EoDataset, NetCDFDataset

# module level logger
LOGGER = logging.getLogger(__name__)


def predict_ERA5(net, ERA5_ds, predictand, batch_size=16, **kwargs):
    # create PyTorch compliant dataset and dataloader instances for model
    # inference
    ds = NetCDFDataset(ERA5_ds, **kwargs)
    dl = DataLoader(ds, batch_size=batch_size, shuffle=False, drop_last=False)

    # device to compute on: use gpu if available
    device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
    LOGGER.info('Predicting on device: {}'.format(device))

    # check if multiple gpus are available
    if torch.cuda.device_count() > 1:
        LOGGER.info('Using {} available GPUs.'.format(
            torch.cuda.device_count()))
        net = torch.nn.DataParallel(net)

    # copy network to target device and set to evaluation mode
    net = net.to(device)
    LOGGER.info('Setting model to evaluation mode ...')
    net.eval()

    # iterate over the reference period
    target = None
    for batch, (inputs, _) in enumerate(dl):

        # send the data to the gpu if available
        inputs = inputs.to(device)

        # calculate network outputs
        with torch.no_grad():
            outputs = net(inputs).cpu().numpy().squeeze()

        # stack predictions
        target = (np.concatenate((target, outputs), axis=0) if batch > 0 else
                  outputs)
        LOGGER.info('Mini-batch: {:d}/{:d}'.format(batch + 1, len(dl)))

    # convert numpy array to xarray.Dataset
    if predictand == 'tas':
        # in case of tas, the netwokr predicts both tasmax and tasmin
        ds = {'tasmax': EoDataset.add_coordinates(target[:, 0, ...].squeeze()),
              'tasmin': EoDataset.add_coordinates(target[:, 1, ...].squeeze())}
    elif predictand == 'pr':
        ds = {'prob': EoDataset.add_coordinates(target[:, 0, ...].squeeze()),

              # amount of precipitation: expected value of gamma distribution
              # pr = shape * scale
              'precipitation': EoDataset.add_coordinates(
                  (np.exp(target[:, 1, ...]) *
                   np.exp(target[:, 2, ...])).squeeze())}
    else:
        # single predictand
        ds = {predictand: EoDataset.add_coordinates(target)}

    # create xarray dataset: dtype=Float32
    ds = xr.Dataset(data_vars=ds,
                    coords={'time': ERA5_ds.time, 'y': ERA5_ds.y,
                            'x': ERA5_ds.x})

    return ds
